/**
 * \file LRU_queue.hh
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date February 2004
 * \brief The header file that defines a LRU_queue object.
 *
 * An LRU_queue is a specific type of Queue that implements the LRU
 * replacement policy.
 **/



// ==================================================================
#if !defined (_LRU_QUEUE_HH)
#define _LRU_QUEUE_HH
// ==================================================================



// ==================================================================
// INCLUDES

#include "Queue.hh"
// ==================================================================



// ==================================================================
/**
 * \brief A LRU_queue organizes and replaces pages by order of first
 *        use.
 **/
class LRU_queue : public Queue {

public:

  /**
   * \brief The constructor.
   * \param memory_size The maximum number of pages managed by the
   *        LRU queue.
   *
   * Prepare an LRU queue by initializing the parent Queue structures.
   **/
  LRU_queue (const unsigned int memory_size);

protected: // METHODS

  /**
   * \brief Respond to a reference to a page already in the
   *        queue.
   * \param referenced_page_number The referenced page that caused the
   *        hit.
   * \param map_iterator An iterator into the map that connects the
   *        referenced page number to its list position.
   *
   * A page in the queue has been referenced.  Move that page to the
   * front of the queue.
   **/
  virtual void handle_hit (const page_number_t referenced_page_number,
			   page_map_t::iterator map_iterator);

  /** 
   * \brief Respond to a reference to a page that is not in the
   *        queue.
   * \param referenced_page_number The referenced page that caused the
   *        miss.
   *
   * A page that is not in the queue has been referenced.  If the
   * queue is full, evict the page that has not been referenced in the
   * longest time, and insert the referenced page at the front of the
   * list.
   **/
  virtual void handle_miss (const page_number_t referenced_page_number);

}; // class LRU_queue
// ==================================================================



// ==================================================================
#endif // _LRU_QUEUE_HH
// ==================================================================
