/**
 * \file Queue.hh
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date February 2004
 * \brief The header file that defines the abstract class Queue.
 *
 * A Queue organizes the pages of a fixed-size memory, into a list,
 * and provides a hash map for constant-time lookup into that list.
 * It implements no replacement policy, and so it is only a abstract
 * class, where subclasses must handle replacement and list
 * re-ordering.
 **/



// ==================================================================
#if !defined (_QUEUE_HH)
#define _QUEUE_HH
// ==================================================================



// ==================================================================
// INCLUDES

#include <list.h>
#include <hash_map.h>
#include "types-and-constants.hh"
// ==================================================================



// ==================================================================
// TYPES

/** \brief A type to order the pages. **/
typedef list<page_number_t> page_list_t;

/**
 * \brief A structure that exists to provide a method of comparison
 *        for the lookup table of pages.
 **/
struct equal_page_number_s {
  bool operator() (const page_number_t& x,
		   const page_number_t& y) const {
    return (x == y);
  }
};

/**
 * \brief A type to provide constant time lookup into the page
 *        list.
 **/
typedef hash_map<page_number_t,
		 page_list_t::iterator,
		 hash<page_number_t>,
		 equal_page_number_s> page_map_t;
// ==================================================================



// ==================================================================
/**
 * \brief A Queue organizes pages into a list and provides an
 *        interface for managing a stream of references.
 **/
class Queue {

public:

  /**
   * \brief The constructor.
   * \param memory_size The maximum number of pages managed by the
   *        Queue.
   *
   * Prepare a Queue by initializing a list and hash map, and
   * recording the maximum number of pages that can be in the queue
   * before replacement is required.
   **/
  Queue (const unsigned int memory_size);

  /**
   * \brief The destructor.
   *
   * Clean up before deleting a queue.  Note that there is nothing to
   * do here, but that a virtual distructor is necessary to support
   * the pure virtual methods and keep the compiler from complaining.
   **/
  virtual ~Queue () {}

  /**
   * \brief Reference a page.
   * \param referenced_page_number The page number being referenced.
   * \return Whether the referenced page was resident.
   **/
  bool touch (const page_number_t referenced_page_number);

protected: // METHODS

  /**
   * \brief Respond to a reference to a page already in the
   *        queue.
   * \param referenced_page_number The referenced page that caused the
   *        hit.
   * \param map_iterator An iterator into the map that connects the
   *        referenced page number to its list position.
   *
   * A page in the queue has been referenced.  Update the page list
   * ordering according to the desired replacement policy.  Note that
   * the parameter provides not only direct access to the correct map
   * entry, but also indirect access to the correct list entry.
   **/
  virtual void handle_hit (const page_number_t referenced_page_number,
			   page_map_t::iterator map_iterator) = 0;

  /** 
   * \brief Respond to a reference to a page that is not in the
   *        queue.
   * \param referenced_page_number The referenced page that caused the
   *        miss.
   *
   * A page that is not in the queue has been referenced.  If the
   * queue is full, evict some page according to the desired
   * replacement policy.  Insert the referenced page into the list and
   * the map.
   **/
  virtual void handle_miss (const page_number_t referenced_page_number)
    = 0;

protected: // DATA

  /** \brief The maximum number of pages that the queue can hold. **/
  const unsigned int memory_size;

  /** \brief The current number of pages held in the queue. **/
  unsigned int pages_held;

  /** \brief The ordered list of page numbers. **/
  page_list_t page_list;

  /** \brief The map from page numbers to list positions. **/
  page_map_t page_map;

}; // class Queue
// ==================================================================



// ==================================================================
#endif // _QUEUE_HH
// ==================================================================
