/**
 * \file deletion.cc
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date February 2004
 * \brief The main file for the deletion trace reducer.
 *
 * Perform an `x-deletion' on a trace, where `x' is some replacement
 * policy.  Given a memory of a particular size (the `reduction
 * memory') managed by policy `x', delete all references to those
 * pages that reside in the reduction memory.
 **/



// ==================================================================
// INCLUDES

#include <stdio.h>
#include <stdlib.h>

#include "types-and-constants.hh"
#include "FIFO_queue.hh"
#include "LRU_queue.hh"
// ==================================================================



// ==================================================================
/**
 * \brief Perform the trace reduction.
 * \param reduction_memory The queue of pages to which references are
 *        ignored.
 *
 * Read each record from the source trace, and discard those
 * references that are to pages in the FIFO queue.
 **/
void
reduce_trace (Queue* reduction_memory) {

  // Read in each record from the source trace.
  page_number_t page_number;
  int result = scanf("%x", &page_number);
  while (result != EOF) {

    // Sanity check: One value should have been interpreted. 
    assert(result == 1);

    // Pass the referenced page to the queue and grab the result of
    // that reference.
    bool was_in_queue = reduction_memory->touch(page_number);

    // Was the reference to a page not in the queue?
    if (!was_in_queue) {

      // It was, so output this reference to the reduced trace.
      printf("%x\n", page_number);

    }

    result = scanf("%x", &page_number);

  }
}
// ==================================================================




// ==================================================================
// Process the arguments used to invoke the program and perform the
// reduction.  Note that the use of stdin and stdout are assumed.
int
main (unsigned int argc, const char** const argv) {

  // Ensure that the right number of arguments were passed.
  if (argc != 3) {

    fprintf(stderr, "Usage: %s\n", argv[0]);
    fprintf(stderr, "       <deletion policy (LRU/FIFO)\n");
    fprintf(stderr, "       <reduction memory size (pages)>\n");
    exit(1);

  }

  // Grab the size of the reduction memory.
  int reduction_memory_size = atol(argv[2]);
  if (reduction_memory_size < 1) {

    fprintf(stderr, "The reduction memory must be at least 1 page\n");
    exit(1);

  }

  // Grab the policy to be used for the reduction memory.
  Queue* reduction_memory = 0;
  if (strcmp("LRU", argv[1]) == 0) {

    reduction_memory = new LRU_queue(reduction_memory_size);

  } else if (strcmp("FIFO", argv[1]) == 0) {

    reduction_memory = new FIFO_queue(reduction_memory_size);

  } else {

    fprintf(stderr, "Unknown queue type: %s\n", argv[1]);
    exit(1);

  }

  // Sanity check.
  assert(reduction_memory != 0);

  // Perform the reduction.
  reduce_trace(reduction_memory);

  return 0;

}
// ==================================================================

